from random import getrandbits
import struct

from purehash._util import pack, unpack


SIZE_MAP = {
    1: "B",
    2: "H",
    4: "I",
    8: "Q",
}

LITTLE_ENDIAN_MAP = {True: "<", False: ">"}


def test_pack_unpack():
    for size in (1, 2, 4, 8):
        for little_endian in (True, False):
            for len_numbers in range(1, 5):
                numbers = tuple(getrandbits(size * 8) for _ in range(len_numbers))
                struct_format = LITTLE_ENDIAN_MAP[little_endian] + (
                    SIZE_MAP[size] * len_numbers
                )

                packed = pack(size, little_endian, *numbers)
                assert packed == struct.pack(struct_format, *numbers)

                unpacked = unpack(size, little_endian, packed)
                assert unpacked == struct.unpack(struct_format, packed)



from __future__ import annotations

from purehash._util import pack, padding, right_rotate, unpack

SHA256_CONSTANTS: tuple[int, ...] = (
    0x428A2F98,
    0x71374491,
    0xB5C0FBCF,
    0xE9B5DBA5,
    0x3956C25B,
    0x59F111F1,
    0x923F82A4,
    0xAB1C5ED5,
    0xD807AA98,
    0x12835B01,
    0x243185BE,
    0x550C7DC3,
    0x72BE5D74,
    0x80DEB1FE,
    0x9BDC06A7,
    0xC19BF174,
    0xE49B69C1,
    0xEFBE4786,
    0x0FC19DC6,
    0x240CA1CC,
    0x2DE92C6F,
    0x4A7484AA,
    0x5CB0A9DC,
    0x76F988DA,
    0x983E5152,
    0xA831C66D,
    0xB00327C8,
    0xBF597FC7,
    0xC6E00BF3,
    0xD5A79147,
    0x06CA6351,
    0x14292967,
    0x27B70A85,
    0x2E1B2138,
    0x4D2C6DFC,
    0x53380D13,
    0x650A7354,
    0x766A0ABB,
    0x81C2C92E,
    0x92722C85,
    0xA2BFE8A1,
    0xA81A664B,
    0xC24B8B70,
    0xC76C51A3,
    0xD192E819,
    0xD6990624,
    0xF40E3585,
    0x106AA070,
    0x19A4C116,
    0x1E376C08,
    0x2748774C,
    0x34B0BCB5,
    0x391C0CB3,
    0x4ED8AA4A,
    0x5B9CCA4F,
    0x682E6FF3,
    0x748F82EE,
    0x78A5636F,
    0x84C87814,
    0x8CC70208,
    0x90BEFFFA,
    0xA4506CEB,
    0xBEF9A3F7,
    0xC67178F2,
)


class SHA256:
    _a: int
    _b: int
    _c: int
    _d: int
    _e: int
    _f: int
    _g: int
    _h: int

    _blocks_processed: int
    _buffer: bytearray

    def __init__(self, message: bytes = b"") -> None:
        self._a = 0x6A09E667
        self._b = 0xBB67AE85
        self._c = 0x3C6EF372
        self._d = 0xA54FF53A
        self._e = 0x510E527F
        self._f = 0x9B05688C
        self._g = 0x1F83D9AB
        self._h = 0x5BE0CD19

        self._blocks_processed = 0
        self._buffer = bytearray()

        self.update(message)

    def _process_block(self, block: bytes) -> None:
        w: list[int] = list(unpack(4, False, block))

        i: int
        s0: int
        s1: int
        for i in range(16, 64):
            s0 = (
                right_rotate(w[i - 15], 7, 32)
                ^ right_rotate(w[i - 15], 18, 32)
                ^ (w[i - 15] >> 3)
            )
            s1 = (
                right_rotate(w[i - 2], 17, 32)
                ^ right_rotate(w[i - 2], 19, 32)
                ^ (w[i - 2] >> 10)
            )
            w.append((w[i - 16] + s0 + w[i - 7] + s1) % (2**32))

        a: int = self._a
        b: int = self._b
        c: int = self._c
        d: int = self._d
        e: int = self._e
        f: int = self._f
        g: int = self._g
        h: int = self._h

        ch: int
        temp0: int
        maj: int
        temp1: int
        for i in range(64):
            s1 = (
                right_rotate(e, 6, 32)
                ^ right_rotate(e, 11, 32)
                ^ right_rotate(e, 25, 32)
            )
            ch = (e & f) ^ ((~e) & g)
            temp0 = (h + s1 + ch + SHA256_CONSTANTS[i] + w[i]) % (2**32)
            s0 = (
                right_rotate(a, 2, 32)
                ^ right_rotate(a, 13, 32)
                ^ right_rotate(a, 22, 32)
            )
            maj = (a & b) ^ (a & c) ^ (b & c)
            temp1 = (s0 + maj) % (2**32)

            h = g
            g = f
            f = e
            e = (d + temp0) % (2**32)
            d = c
            c = b
            b = a
            a = (temp0 + temp1) % (2**32)

        self._a = (self._a + a) % (2**32)
        self._b = (self._b + b) % (2**32)
        self._c = (self._c + c) % (2**32)
        self._d = (self._d + d) % (2**32)
        self._e = (self._e + e) % (2**32)
        self._f = (self._f + f) % (2**32)
        self._g = (self._g + g) % (2**32)
        self._h = (self._h + h) % (2**32)

    def update(self, message: bytes) -> None:
        self._buffer += message

        for _ in range(len(self._buffer) // 64):
            self._process_block(self._buffer[:64])

            self._blocks_processed += 1
            self._buffer = self._buffer[64:]

    def digest(self) -> bytes:
        # Save state.
        a: int = self._a
        b: int = self._b
        c: int = self._c
        d: int = self._d
        e: int = self._e
        f: int = self._f
        g: int = self._g
        h: int = self._h

        buffer_length: int = len(self._buffer)
        self._buffer += padding(
            (self._blocks_processed * 64) + buffer_length, 64, 8, False
        )

        self._process_block(self._buffer[:64])

        if len(self._buffer) == 128:
            self._process_block(self._buffer[64:])

        result: bytes = pack(
            4,
            False,
            self._a,
            self._b,
            self._c,
            self._d,
            self._e,
            self._f,
            self._g,
            self._h,
        )

        # Restore state.
        self._a = a
        self._b = b
        self._c = c
        self._d = d
        self._e = e
        self._f = f
        self._g = g
        self._h = h

        # Restore buffer.
        self._buffer = self._buffer[:buffer_length]

        return result

    def hexdigest(self) -> str:
        return self.digest().hex()


SHA512_CONSTANTS: tuple[int, ...] = (
    0x428A2F98D728AE22,
    0x7137449123EF65CD,
    0xB5C0FBCFEC4D3B2F,
    0xE9B5DBA58189DBBC,
    0x3956C25BF348B538,
    0x59F111F1B605D019,
    0x923F82A4AF194F9B,
    0xAB1C5ED5DA6D8118,
    0xD807AA98A3030242,
    0x12835B0145706FBE,
    0x243185BE4EE4B28C,
    0x550C7DC3D5FFB4E2,
    0x72BE5D74F27B896F,
    0x80DEB1FE3B1696B1,
    0x9BDC06A725C71235,
    0xC19BF174CF692694,
    0xE49B69C19EF14AD2,
    0xEFBE4786384F25E3,
    0x0FC19DC68B8CD5B5,
    0x240CA1CC77AC9C65,
    0x2DE92C6F592B0275,
    0x4A7484AA6EA6E483,
    0x5CB0A9DCBD41FBD4,
    0x76F988DA831153B5,
    0x983E5152EE66DFAB,
    0xA831C66D2DB43210,
    0xB00327C898FB213F,
    0xBF597FC7BEEF0EE4,
    0xC6E00BF33DA88FC2,
    0xD5A79147930AA725,
    0x06CA6351E003826F,
    0x142929670A0E6E70,
    0x27B70A8546D22FFC,
    0x2E1B21385C26C926,
    0x4D2C6DFC5AC42AED,
    0x53380D139D95B3DF,
    0x650A73548BAF63DE,
    0x766A0ABB3C77B2A8,
    0x81C2C92E47EDAEE6,
    0x92722C851482353B,
    0xA2BFE8A14CF10364,
    0xA81A664BBC423001,
    0xC24B8B70D0F89791,
    0xC76C51A30654BE30,
    0xD192E819D6EF5218,
    0xD69906245565A910,
    0xF40E35855771202A,
    0x106AA07032BBD1B8,
    0x19A4C116B8D2D0C8,
    0x1E376C085141AB53,
    0x2748774CDF8EEB99,
    0x34B0BCB5E19B48A8,
    0x391C0CB3C5C95A63,
    0x4ED8AA4AE3418ACB,
    0x5B9CCA4F7763E373,
    0x682E6FF3D6B2B8A3,
    0x748F82EE5DEFB2FC,
    0x78A5636F43172F60,
    0x84C87814A1F0AB72,
    0x8CC702081A6439EC,
    0x90BEFFFA23631E28,
    0xA4506CEBDE82BDE9,
    0xBEF9A3F7B2C67915,
    0xC67178F2E372532B,
    0xCA273ECEEA26619C,
    0xD186B8C721C0C207,
    0xEADA7DD6CDE0EB1E,
    0xF57D4F7FEE6ED178,
    0x06F067AA72176FBA,
    0x0A637DC5A2C898A6,
    0x113F9804BEF90DAE,
    0x1B710B35131C471B,
    0x28DB77F523047D84,
    0x32CAAB7B40C72493,
    0x3C9EBE0A15C9BEBC,
    0x431D67C49C100D4C,
    0x4CC5D4BECB3E42B6,
    0x597F299CFC657E2A,
    0x5FCB6FAB3AD6FAEC,
    0x6C44198C4A475817,
)


class SHA512:
    _a: int
    _b: int
    _c: int
    _d: int
    _e: int
    _f: int
    _g: int
    _h: int

    _blocks_processed: int
    _buffer: bytearray

    def __init__(self, message: bytes = b"") -> None:
        self._a = 0x6A09E667F3BCC908
        self._b = 0xBB67AE8584CAA73B
        self._c = 0x3C6EF372FE94F82B
        self._d = 0xA54FF53A5F1D36F1
        self._e = 0x510E527FADE682D1
        self._f = 0x9B05688C2B3E6C1F
        self._g = 0x1F83D9ABFB41BD6B
        self._h = 0x5BE0CD19137E2179

        self._blocks_processed = 0
        self._buffer = bytearray()

        self.update(message)

    def _process_block(self, block: bytes) -> None:
        w: list[int] = list(unpack(8, False, block))

        i: int
        s0: int
        s1: int
        for i in range(16, 80):
            s0 = (
                right_rotate(w[i - 15], 1, 64)
                ^ right_rotate(w[i - 15], 8, 64)
                ^ (w[i - 15] >> 7)
            )
            s1 = (
                right_rotate(w[i - 2], 19, 64)
                ^ right_rotate(w[i - 2], 61, 64)
                ^ (w[i - 2] >> 6)
            )
            w.append((w[i - 16] + s0 + w[i - 7] + s1) % (2**64))

        a: int = self._a
        b: int = self._b
        c: int = self._c
        d: int = self._d
        e: int = self._e
        f: int = self._f
        g: int = self._g
        h: int = self._h

        ch: int
        temp0: int
        maj: int
        temp1: int
        for i in range(80):
            s1 = (
                right_rotate(e, 14, 64)
                ^ right_rotate(e, 18, 64)
                ^ right_rotate(e, 41, 64)
            )
            ch = (e & f) ^ ((~e) & g)
            temp0 = (h + s1 + ch + SHA512_CONSTANTS[i] + w[i]) % (2**64)
            s0 = (
                right_rotate(a, 28, 64)
                ^ right_rotate(a, 34, 64)
                ^ right_rotate(a, 39, 64)
            )
            maj = (a & b) ^ (a & c) ^ (b & c)
            temp1 = (s0 + maj) % (2**64)

            h = g
            g = f
            f = e
            e = (d + temp0) % (2**64)
            d = c
            c = b
            b = a
            a = (temp0 + temp1) % (2**64)

        self._a = (self._a + a) % (2**64)
        self._b = (self._b + b) % (2**64)
        self._c = (self._c + c) % (2**64)
        self._d = (self._d + d) % (2**64)
        self._e = (self._e + e) % (2**64)
        self._f = (self._f + f) % (2**64)
        self._g = (self._g + g) % (2**64)
        self._h = (self._h + h) % (2**64)

    def update(self, message: bytes) -> None:
        self._buffer += message

        for _ in range(len(self._buffer) // 128):
            self._process_block(self._buffer[:128])

            self._blocks_processed += 1
            self._buffer = self._buffer[128:]

    def digest(self) -> bytes:
        # Save state.
        a: int = self._a
        b: int = self._b
        c: int = self._c
        d: int = self._d
        e: int = self._e
        f: int = self._f
        g: int = self._g
        h: int = self._h

        buffer_length: int = len(self._buffer)
        self._buffer += padding(
            (self._blocks_processed * 128) + buffer_length, 128, 16, False
        )

        self._process_block(self._buffer[:128])

        if len(self._buffer) == 256:
            self._process_block(self._buffer[128:])

        result: bytes = pack(
            8,
            False,
            self._a,
            self._b,
            self._c,
            self._d,
            self._e,
            self._f,
            self._g,
            self._h,
        )

        # Restore state.
        self._a = a
        self._b = b
        self._c = c
        self._d = d
        self._e = e
        self._f = f
        self._g = g
        self._h = h

        # Restore buffer.
        self._buffer = self._buffer[:buffer_length]

        return result

    def hexdigest(self) -> str:
        return self.digest().hex()



from __future__ import annotations

from purehash._util import left_rotate, pack, padding, unpack


class SHA1:
    _a: int
    _b: int
    _c: int
    _d: int
    _e: int

    _blocks_processed: int
    _buffer: bytearray

    def __init__(self, message: bytes = b""):
        self._a = 0x67452301
        self._b = 0xEFCDAB89
        self._c = 0x98BADCFE
        self._d = 0x10325476
        self._e = 0xC3D2E1F0

        self._blocks_processed = 0
        self._buffer = bytearray()

        self.update(message)

    def _process_block(self, block: bytes) -> None:
        w: list[int] = list(unpack(4, False, block))

        for i in range(16, 80):
            w.append(left_rotate(w[i - 3] ^ w[i - 8] ^ w[i - 14] ^ w[i - 16], 1, 32))

        a: int = self._a
        b: int = self._b
        c: int = self._c
        d: int = self._d
        e: int = self._e

        f: int
        k: int
        temp: int
        for i in range(80):
            if i < 20:
                f = (b & c) | ((~b) & d)
                k = 0x5A827999
            elif i < 40:
                f = b ^ c ^ d
                k = 0x6ED9EBA1
            elif i < 60:
                f = (b & c) | (b & d) | (c & d)
                k = 0x8F1BBCDC
            else:
                f = b ^ c ^ d
                k = 0xCA62C1D6

            temp = (left_rotate(a, 5, 32) + f + e + k + w[i]) % (2**32)
            e = d
            d = c
            c = left_rotate(b, 30, 32)
            b = a
            a = temp

        self._a = (self._a + a) % (2**32)
        self._b = (self._b + b) % (2**32)
        self._c = (self._c + c) % (2**32)
        self._d = (self._d + d) % (2**32)
        self._e = (self._e + e) % (2**32)

    def update(self, message: bytes) -> None:
        self._buffer += message

        for _ in range(len(self._buffer) // 64):
            self._process_block(self._buffer[:64])

            self._blocks_processed += 1
            self._buffer = self._buffer[64:]

    def digest(self) -> bytes:
        # Save state.
        a: int = self._a
        b: int = self._b
        c: int = self._c
        d: int = self._d
        e: int = self._e

        buffer_length: int = len(self._buffer)
        self._buffer += padding(
            (self._blocks_processed * 64) + buffer_length, 64, 8, False
        )

        self._process_block(self._buffer[:64])

        if len(self._buffer) == 128:
            self._process_block(self._buffer[64:])

        result: bytes = pack(4, False, self._a, self._b, self._c, self._d, self._e)

        # Restore state.
        self._a = a
        self._b = b
        self._c = c
        self._d = d
        self._e = e

        # Restore buffer.
        self._buffer = self._buffer[:buffer_length]

        return result

    def hexdigest(self) -> str:
        return self.digest().hex()



from hashlib import sha256, sha512

from purehash._sha2 import SHA256, SHA512
from purehash._util import random_tests


def test_sha256():
    random_tests(sha256, SHA256, (55, 56, 57, 63, 64, 65))


def test_sha512():
    random_tests(sha512, SHA512, (111, 112, 113, 127, 128, 129))



from __future__ import annotations

from purehash._util import left_rotate, pack, padding, unpack

SHIFTS: tuple[int, ...] = (
    7,
    12,
    17,
    22,
    7,
    12,
    17,
    22,
    7,
    12,
    17,
    22,
    7,
    12,
    17,
    22,
    5,
    9,
    14,
    20,
    5,
    9,
    14,
    20,
    5,
    9,
    14,
    20,
    5,
    9,
    14,
    20,
    4,
    11,
    16,
    23,
    4,
    11,
    16,
    23,
    4,
    11,
    16,
    23,
    4,
    11,
    16,
    23,
    6,
    10,
    15,
    21,
    6,
    10,
    15,
    21,
    6,
    10,
    15,
    21,
    6,
    10,
    15,
    21,
)
SINES: tuple[int, ...] = (
    0xD76AA478,
    0xE8C7B756,
    0x242070DB,
    0xC1BDCEEE,
    0xF57C0FAF,
    0x4787C62A,
    0xA8304613,
    0xFD469501,
    0x698098D8,
    0x8B44F7AF,
    0xFFFF5BB1,
    0x895CD7BE,
    0x6B901122,
    0xFD987193,
    0xA679438E,
    0x49B40821,
    0xF61E2562,
    0xC040B340,
    0x265E5A51,
    0xE9B6C7AA,
    0xD62F105D,
    0x02441453,
    0xD8A1E681,
    0xE7D3FBC8,
    0x21E1CDE6,
    0xC33707D6,
    0xF4D50D87,
    0x455A14ED,
    0xA9E3E905,
    0xFCEFA3F8,
    0x676F02D9,
    0x8D2A4C8A,
    0xFFFA3942,
    0x8771F681,
    0x6D9D6122,
    0xFDE5380C,
    0xA4BEEA44,
    0x4BDECFA9,
    0xF6BB4B60,
    0xBEBFBC70,
    0x289B7EC6,
    0xEAA127FA,
    0xD4EF3085,
    0x04881D05,
    0xD9D4D039,
    0xE6DB99E5,
    0x1FA27CF8,
    0xC4AC5665,
    0xF4292244,
    0x432AFF97,
    0xAB9423A7,
    0xFC93A039,
    0x655B59C3,
    0x8F0CCC92,
    0xFFEFF47D,
    0x85845DD1,
    0x6FA87E4F,
    0xFE2CE6E0,
    0xA3014314,
    0x4E0811A1,
    0xF7537E82,
    0xBD3AF235,
    0x2AD7D2BB,
    0xEB86D391,
)


class MD5:
    _a: int
    _b: int
    _c: int
    _d: int

    _blocks_processed: int
    _buffer: bytearray

    def __init__(self, message: bytes = b"") -> None:
        self._a = 0x67452301
        self._b = 0xEFCDAB89
        self._c = 0x98BADCFE
        self._d = 0x10325476

        self._blocks_processed = 0
        self._buffer = bytearray()

        self.update(message)

    def _process_block(self, block: bytes) -> None:
        m: tuple[int, ...] = unpack(4, True, block)

        a: int = self._a
        b: int = self._b
        c: int = self._c
        d: int = self._d

        i: int
        f: int
        g: int
        for i in range(64):
            if i < 16:
                f = (b & c) | ((~b) & d)
                g = i
            elif i < 32:
                f = (d & b) | ((~d) & c)
                g = (5 * i + 1) % 16
            elif i < 48:
                f = b ^ c ^ d
                g = (3 * i + 5) % 16
            else:
                f = c ^ (b | (~d))
                g = (7 * i) % 16

            f = (f + a + SINES[i] + m[g]) % (2**32)
            a = d
            d = c
            c = b
            b = b + left_rotate(f, SHIFTS[i], 32)

        self._a = (self._a + a) % (2**32)
        self._b = (self._b + b) % (2**32)
        self._c = (self._c + c) % (2**32)
        self._d = (self._d + d) % (2**32)

    def update(self, message: bytes) -> None:
        self._buffer += message

        for _ in range(len(self._buffer) // 64):
            self._process_block(self._buffer[:64])

            self._blocks_processed += 1
            self._buffer = self._buffer[64:]

    def digest(self) -> bytes:
        # Save state.
        a: int = self._a
        b: int = self._b
        c: int = self._c
        d: int = self._d

        buffer_length: int = len(self._buffer)
        self._buffer += padding(
            (self._blocks_processed * 64) + buffer_length, 64, 8, True
        )

        self._process_block(self._buffer[:64])

        if len(self._buffer) == 128:
            self._process_block(self._buffer[64:])

        result: bytes = pack(4, True, self._a, self._b, self._c, self._d)

        # Restore state.
        self._a = a
        self._b = b
        self._c = c
        self._d = d

        # Restore buffer.
        self._buffer = self._buffer[:buffer_length]

        return result

    def hexdigest(self) -> str:
        return self.digest().hex()



from __future__ import annotations
from random import choice, getrandbits
from typing import Any


def left_rotate(number: int, rotation: int, bits: int) -> int:
    rotation = rotation % bits
    return ((number << rotation) % (2**bits)) | (number >> (bits - rotation))


def right_rotate(number: int, rotation: int, bits: int) -> int:
    rotation = rotation % bits
    return left_rotate(number, bits - rotation, bits)


def pack(size: int, little_endian: bool, *args: int) -> bytes:
    result: bytearray = bytearray()

    number: int
    for number in args:
        packed: bytearray = bytearray()
        for _ in range(size):
            byte: int = number % (2**8)
            number >>= 8
            if little_endian:
                packed.append(byte)
            else:
                packed.insert(0, byte)

        result += packed

    return bytes(result)


def unpack(size: int, little_endian: bool, bytes_: bytes) -> tuple[int, ...]:
    assert len(bytes_) % size == 0, "Length of bytes_ must be a multiple of size."

    result: list[int] = []

    i: int
    for i in range(0, len(bytes_), size):
        packed: bytearray = bytearray(bytes_[i : i + size])
        number: int = 0
        for _ in range(size):
            number <<= 8

            byte: int
            if little_endian:
                byte = packed.pop()
            else:
                byte = packed.pop(0)

            number += byte

        result.append(number)

    return tuple(result)


def padding(
    length: int, block_size: int, length_size: int, length_little_endian: bool
) -> bytes:
    padding_: bytearray = bytearray(b"\x80")

    length_mod: int = length % block_size
    if length_mod < (block_size - (block_size // 8)):
        padding_ += b"\x00" * (block_size - (block_size // 8) - 1 - length_mod)
    else:
        padding_ += b"\x00" * (block_size * 2 - (block_size // 8) - 1 - length_mod)

    padding_ += pack(length_size, length_little_endian, length * 8)

    return bytes(padding_)


def random_tests(x: Any, y: Any, problem_lengths: tuple[int, ...]) -> None:
    lengths: tuple[int, ...] = (0, 1) + problem_lengths

    length: int
    r: bytes
    for _ in range(16):
        length = choice(lengths)
        r = bytes(getrandbits(8) for _ in range(length))

        x_: Any = x(r)
        y_: Any = y(r)

        assert x_.digest() == y_.digest()
        assert x_.hexdigest() == y_.hexdigest()

        for _ in range(16):
            length = choice(lengths)
            r = bytes(getrandbits(8) for _ in range(length))

            x_.update(r)
            y_.update(r)

            assert x_.digest() == y_.digest()
            assert x_.hexdigest() == y_.hexdigest()



from setuptools import find_packages, setup
from pathlib import Path

project_directory = Path(__file__).parent.resolve()

setup(
    name="purehash",
    version="1.0.0",
    description="Pure Python implementations of common hashing algorithms.",
    long_description=(project_directory / "README.md").read_text("utf-8"),
    long_description_content_type="text/markdown",
    url="https://github.com/oshawk/purehash",
    author="Oshawk",
    classifiers=[
        "Intended Audience :: Developers",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3 :: Only",
    ],
    keywords="hash, cryptography",
    packages=find_packages(),
    python_requires=">=3.7, <4",
    extras_require={
        "dev": ["pre-commit", "black", "mypy", "twine"],
        "test": ["pytest"],
    },
)
